package org.wikipedia.dataclient.okhttp
import android.os.Build
import android.util.Log
import okhttp3.OkHttpClient
import java.net.InetAddress
import java.net.Socket
import java.security.KeyStore
import javax.net.ssl.SNIHostName
import javax.net.ssl.SSLSocket
import javax.net.ssl.SSLSocketFactory
import javax.net.ssl.TrustManagerFactory
import javax.net.ssl.X509TrustManager
private val sslSocketFactory = SSLSocketFactory.getDefault() as SSLSocketFactory
private const val FRONTING_DOMAIN = "m.mediawiki.org"
private val censoredDomain = listOf(
    "wikipedia.org",
    "wikinews.org",
    "wikiquote.org"
)
private fun String.isCensored() = censoredDomain.any { this == it || this.endsWith(".$it") }
object OkHttpSSLSocketFactory : SSLSocketFactory() {
    override fun getDefaultCipherSuites(): Array<String> = sslSocketFactory.defaultCipherSuites
    override fun getSupportedCipherSuites(): Array<String> = sslSocketFactory.supportedCipherSuites
    // https://gist.github.com/vvb2060/d51fefce4a269d6becfe298d7b588ff1
    override fun createSocket(s: Socket, host: String, port: Int, autoClose: Boolean): Socket {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
            val socket = sslSocketFactory.createSocket(s, host, port, autoClose) as SSLSocket
            val params = socket.sslParameters
            params.serverNames = listOf(SNIHostName(FRONTING_DOMAIN)).takeIf { host.isCensored() || host in WIKIMEDIA_DOH_URL }
            socket.sslParameters = params
            return socket
        } else {
            val address = s.inetAddress.hostAddress.takeIf { host.isCensored() || host in WIKIMEDIA_DOH_URL }
            Log.d("OkHttSSLSocketFactory", "Host: $host Address: $address")
            return sslSocketFactory.createSocket(s, address ?: host, port, autoClose) as SSLSocket
        }
    }
    override fun createSocket(host: String, port: Int): Socket = sslSocketFactory.createSocket(host, port)
    override fun createSocket(host: String, port: Int, localHost: InetAddress, localPort: Int): Socket = sslSocketFactory.createSocket(host, port, localHost, localPort)
    override fun createSocket(host: InetAddress, port: Int): Socket = sslSocketFactory.createSocket(host, port)
    override fun createSocket(address: InetAddress, port: Int, localAddress: InetAddress, localPort: Int): Socket = sslSocketFactory.createSocket(address, port, localAddress, localPort)
}
fun OkHttpClient.Builder.install(sslSocketFactory: SSLSocketFactory) = apply {
    val factory = TrustManagerFactory.getInstance(TrustManagerFactory.getDefaultAlgorithm())!!
    factory.init(null as KeyStore?)
    val manager = factory.trustManagers!!
    val trustManager = manager.filterIsInstance<X509TrustManager>().first()
    sslSocketFactory(sslSocketFactory, trustManager)
}
